using System;
using System.Text;
using System.Security.Cryptography;

namespace JEBKeyMaker
{
    public class KeyMaker
    {
        private const int CONST1 = -168884021;
        private const int CONST2 = 287454020;
        private const int CONST3 = 196640183;
        private const int CONST4 = 1432778632;

        private long machineId;

        public KeyMaker(long machineId)
        {
            this.machineId = machineId;
        }

        public bool isValidKey(String key, int[] outDate)
        {
            if (key != null && key.Length != 0)
            {
                key = key.Trim();
                int date = 0;
                int index = key.IndexOf('Z');
                if (index >= 0)
                {
                    String second = key.Substring(index + 1);
                    if (second.Length < 2)
                    {
                        return false;
                    }

                    String dataPart = second.Substring(0, second.Length - 1);
                    String hashPart = second.Substring(second.Length - 1);

                    int dataVal;
                    try
                    {
                        dataVal = int.Parse(dataPart);
                        int hashVal = int.Parse(hashPart);
                        if (this.dateHash(dataVal) != hashVal)
                        {
                            return false;
                        }
                    }
                    catch (Exception)
                    {
                        return false;
                    }

                    date = dataVal ^ 1450416845;
                    key = key.Substring(0, index);
                }

                try
                {
                    long keyVal = long.Parse(key);
                    if (!this.isValidKey(keyVal))
                    {
                        return false;
                    }
                }
                catch (Exception)
                {
                    return false;
                }

                if (outDate != null && outDate.Length >= 1)
                {
                    outDate[0] = date;
                }

                return true;
            }
            else
            {
                return false;
            }
        }
        
        private int dateHash(int dataVal)
        {
            int hash;
            for (hash = 0; dataVal > 0; dataVal >>= 4)
            {
                hash += dataVal & 15;
            }

            return hash % 10;
        }

        private static byte[] sha256Hash(byte[] buffer)
        {
            SHA256 sha = new SHA256Managed();
            return sha.ComputeHash(buffer);
        }

        public static String toHexString(byte[] buffer)
        {
            StringBuilder sb = new StringBuilder();

            for (int i = 0; i < buffer.Length; ++i)
            {
                sb.Append(String.Format("{0:X2}", buffer[i]));
            }

            return sb.ToString();
        }

        public String getValidKey(int date)
        {
            int lowId = (int)(this.machineId & -1L);
            int hiId = (int)(this.machineId >> 32 & -1L);

            int lowVal = lowId + CONST1 + CONST2 & -1;
            int hiVal = hiId - CONST3 + CONST4 & int.MaxValue;

            long keyVal = (long)(((ulong)hiVal << 32) | (uint)lowVal);

            int lowKey = (int)(keyVal & -1L);
            int hiKey = (int)(keyVal >> 32 & -1L);

            if (lowVal == lowKey && hiVal == hiKey)
            {
                String[] blackHashList = new String[] { "f876add9497b7e2745266f83eebf208b64650fb00e71794892890b42d3e890da", "1020e406024705930a03818381b2379a64929bd068cc96b6e256562c3dc57058", "f876add9497b7e2745266f83eebf208b64650fb00e71794892890b42d3e890da", "cb631c8d21bdc0c7b75e13c423c00a3a5cfe10782921837453ab1993b04410dc", "82bb76b943b8ed15f70992a2830f826637a307c0aace1d1a4e188085169d6e91", "eea85229acd2559ca62f013c81c5f3c2880adc880b8f8e9673eb6926de3d61bb", "a005a5ca6874292a351a4329f1fdaaaf90ea1eda4e002e1179897c7b1f6615c3", "e68104e9daf9509d03be7561a6bbb5afdfe063b595c1423ce2e51ac1dbd345cb", "0df0317ef37bb52d315c12d07011782345bc99a5f9643e21485a36b1b62723a9", "80e6fce9d2df9a1b89317b3bfd9c5b97c8c7a7e90ef3fdf18a7d226994fea927" };
                String keyHash = String.Format("{0}:{1}:{2}", 844563159190538443L, this.machineId, keyVal);
                keyHash = toHexString(sha256Hash(Encoding.Default.GetBytes(keyHash))).ToLower();
                int len = blackHashList.Length;

                for (int i = 0; i < len; ++i)
                {
                    String blackHash = blackHashList[i];
                    if (keyHash.Equals(blackHash))
                    {
                        return "";
                    }
                }

                int dataVal = date ^ 1450416845;
                int hashVal = dateHash(dataVal);
                return String.Format("{0}Z{1}{2}", keyVal, dataVal, hashVal);
            }
            else
            {
                return "";
            }
        }
        
        private bool isValidKey(long keyVal)
        {
            int lowId = (int)(this.machineId & -1L);
            int hiId = (int)(this.machineId >> 32 & -1L);
            int lowKey = (int)(keyVal & -1L);
            int hiKey = (int)(keyVal >> 32 & -1L);
            int lowVal = lowId + CONST1 + CONST2 & -1;
            int hiVal = hiId - CONST3 + CONST4 & int.MaxValue;
            if (lowVal == lowKey && hiVal == hiKey)
            {
                String[] blackHashList = new String[] { "f876add9497b7e2745266f83eebf208b64650fb00e71794892890b42d3e890da", "1020e406024705930a03818381b2379a64929bd068cc96b6e256562c3dc57058", "f876add9497b7e2745266f83eebf208b64650fb00e71794892890b42d3e890da", "cb631c8d21bdc0c7b75e13c423c00a3a5cfe10782921837453ab1993b04410dc", "82bb76b943b8ed15f70992a2830f826637a307c0aace1d1a4e188085169d6e91", "eea85229acd2559ca62f013c81c5f3c2880adc880b8f8e9673eb6926de3d61bb", "a005a5ca6874292a351a4329f1fdaaaf90ea1eda4e002e1179897c7b1f6615c3", "e68104e9daf9509d03be7561a6bbb5afdfe063b595c1423ce2e51ac1dbd345cb", "0df0317ef37bb52d315c12d07011782345bc99a5f9643e21485a36b1b62723a9", "80e6fce9d2df9a1b89317b3bfd9c5b97c8c7a7e90ef3fdf18a7d226994fea927", "63a7e476be6fff6bf20b028813e79335a08c875dd14539f60b029d800693f275", "a5d4d41d6aabc6ad075bb0ac46cbca37733feb85b8e35d4488516132f5c03e01", "898b50a1bde2f3ed15cada4881f2b374318adc138f930008a283cac889a27535", "956db5f7439ca409b01899761582da45df30e41dd2c637f6c186d936ecab3a20", "91ca27de06ed8c01cdc5529efc7577e48a6ceb5ed18b6ca6ae7c23e3a160748d", "7397d231e6401e9c3fcc83d9ccf3ab9af97ab0c6f48f661373dab22092f80ecd", "e4901f636091a674178990368eba7aee91844323fbbe7263301a23656c27f249", "75c2dcffd1abaac7d9f193742714a34bbc41bb6612197fc4a2045b76a631ff57", "c27f7eaed9b55c71248d50d7cc6c8bc5495351f5ee30a41d136f4874556aaa91", "137cca55c3a959e0d2f78d06c357e29b62fbe5fdd8bcc0f035369c1b9227c3bf", "f5776fa1c8ae81baca732ebb397ad6c47118a0b824f3705d1fb83963ef9515bb", "a28fd427cb48bb26e68412b96814891c9f6975eaf9958ee8e9b7bd154487be18", "6daf99f97ac6f2b10dd4efc3108d546ffa34f6e485bf8ad5364b03912584d192", "b60da82d76ec4adb169e1f36efc8b673d45e8eb37695206d69ef93d009a3bba5", "427bca7d6f73da411d1d0f3c5cd31fdeb54196255a0d2ac27aacca4ab3ce8bbd", "d34934669ad50c6ad664478c51ce6a5cca1e10ff74f98348e0e393f902060064", "55c8bae9965eba2c785a108e653720520022804592a72c31930a6b640db8bba0", "5c33c7371fb983c8cc908f338b46b7fe570e6efc6bde53002e15e7fd2ac3750e", "e109953e4990b0ff2b734a6348edd46a5eb1ccc32d09d105e14da16a2875dab4", "8ea1e3b858e698c71122313284133826ae48b80e89c69ecb30f1213c0975c2e7", "2fa7680b7c8ad72b46790fb45128aed196f16ab72a209c2a79f4ad7475e57f77", "d2173167ecdd208538609af7176f40c90d817abae228c30f96f6b13065530d6c", "ce53ce3f4cc2219c74f31779fdc967a985551db9615542e13bbdf947927ce6e3", "117b4ee39eda4a21e64fbef54ec573c2b0f01cc0f04f6af190f1d395cf1bed14", "849f48b0c41e1cc8fef120b2333a1ad43162615f50b87a74331b849e109ca4bc"};
                String keyHash = String.Format("{0}:{1}:{2}", 844563159190538443L, this.machineId, keyVal);
                keyHash = toHexString(sha256Hash(Encoding.Default.GetBytes(keyHash))).ToLower();
                int len = blackHashList.Length;

                for (int i = 0; i < len; ++i)
                {
                    String blackHash = blackHashList[i];
                    if (keyHash.Equals(blackHash))
                    {
                        return false;
                    }
                }

                return true;
            }
            else
            {
                return false;
            }
        }
    }

}
